<?php

require_once "../lib/fileutil.php";


/**
 * This is any type of arbitrary file, generally uploaded by a user.
 * Its primary capability is to be downloaded again with a correctly
 * set mimetype.
 */
class GeneralFile  {
    function GeneralFile() {
    }

    function fillFromDB($db, $file_id) {
        $q = "SELECT *
              FROM files LEFT JOIN genfiles ON files.file_id = genfiles.file_id
              WHERE genfiles.file_id = $file_id";
        
        $this->fillFromRow($db->getRow($q));
    }
    function fillFromRow($docRow) {
        $this->unix_when_created = strtotime($docRow['when_created']);
        foreach($docRow as $k => $v) {
            $this->$k = $v;
        }
    }
    function fillFromInput($origfilename, $mimetype) {
        $this->filename = $this->makeSafeFN($origfilename);
        $this->mimetype = $mimetype;
    }

    /**
     * STATIC
     * Gets rid of dangerous characters in filenames
     */
    function makeSafeFN($untrusted_filename) {
        $n = $untrusted_filename;
        

        // These characters are not allowed on windows:   / \ : * ? " < > |
        // (according to winxp's popup bubble when renaming a file)

        // Unix technically allows any character except for /
        // but avoiding the windows ones is a good idea

        // I'm not sure on mac's rules.  it uses some unix rules,
        // but they also have confusing kludges to be backwards compatible
        // with the old pre-os x mac filesystem (:'s in unix filenames
        // are displayed in the UI as /'s, or something like that)

        // Therefore, I'm willingto bet that windows is the most restrictive,
        // so we should just follow that.

        // There are many more potential rules.
        // e.g. http://www.boost.org/libs/filesystem/doc/portability_guide.htm
        
        $n = str_replace('/', '_', $n);
        $n = str_replace('\\', '_', $n);
        $n = str_replace(':', '_', $n);
        $n = str_replace('*', '_', $n);
        $n = str_replace('?', '_', $n);
        $n = str_replace('"', "'", $n);
        $n = str_replace('<', '_', $n);
        $n = str_replace('>', '_', $n);
        $n = str_replace('|', '_', $n);

        //This is just so no ugly %20's or +'s in web addresses
        //It might not be a good idea, since many people use spaces
        //in their filenames, and underscores are hard to see when url's
        //are underlined.
        $n = preg_replace('/ /', '_', $n);
        
        return $n;
    }

    function getFilename() {
        return $this->filename;
    }

    function getFullFilename() {
        global $CURGROUP;
        $ffn  = $CURGROUP->getDataDir()."/filestore/". $this->file_id;
        $ffn .= "/". $this->filename;
        return $ffn;
    }
/* RESTRUCTURE: obsolete (?)
    function getUrlInternalFullFilename() {
        global $CURGROUP;
        $fn  = $CURGROUP->getDataDir()."/files/".$this->file_id;
        $fn .= "/".$this->filename;
        return $fn;
    }
*/
    function getDownloadUrl() {
        global $CURGROUP;
        $url  = $CURGROUP->getUrlRoot() ."/files/". $this->file_id;
        $url .= "/".urlencode($this->filename);
        return $url;
    }
    function getMimeType() {
        return $this->mimetype;
    }
    function hasReadPerm($db, $username) {
        return true; //TODO implement
    }
    function hasWritePerm($db, $username) {
        return false; //TODO implement (and design!) too
    }
    
    //Dump contents to stdout - pretend we're the file.
    //Content-Type header should've been set before running this!
    function printSelfFromFilename($filename) {
        //ignore $filename, only one choice...
        $fp = fopen($this->getFullFilename(), 'r');
        fpassthru($fp);
        fclose($fp);
    }

    /**
     * Stores the new GeneralFile on the filesystem, from an upload.
     */
    function storeFromUpload($tmp_name) {
        assert(isset($this->filename));
        $path = dirname($this->getFullFilename());
        mkdir_default_mode($path);

        move_uploaded_file($tmp_name, $this->getFullFilename());

    }

    /**
     * Creates database entries for the new GeneralFile
     */
    function storeAsNew($db) {
        assert(isset($this->filename));
        
        $q = "INSERT files VALUES (null, NOW(), 'genfile')";
        $db->query($q);
        $this->file_id = $db->getOne("SELECT LAST_INSERT_ID()");
        $q  = "INSERT genfiles VALUES (".$this->file_id .",";
        $q .=                        "'".$this->filename."',";
        $q .=                        "'".$this->mimetype."')";
        $db->query($q);
    }

    function viewAsBrief() {
        echo "<a href=".$this->getDownloadUrl().">";
        echo "[Click to view]";
        echo "</a>";
    }
    function viewAsEmbedded() {
        echo "<a href=".$this->getDownloadUrl().">";
        echo "[Click to view]";
        echo "</a>";

    }
    function viewAsDownloadUrl() {
        echo "<a href=".$this->getDownloadUrl().">";
        echo $this->getDownloadUrl();
        echo "</a>";
    }
}


function GeneralFile_test()
{
    echo GeneralFile::makeSafeFN("hello world!");
    echo "\n";
}
//GeneralFile_test();