<?
/**
 * @return e.g. 2004-02-12T15:19:21+00:00
 * ...this is supposed to be the universal standard for representing
 * dates/times.
 *
 * in php 5 this is just date('c'), but we don't assume php 5
 *
 * but unfortunately, in php 4 at least, strtotime() doesnt know how to parse
 * strict iso8601 dates like this.
 */
function datetimeFormatIso8601($time)
{
    return date('Y-m-d\TH:i:sO');
}

/**
 * @return e.g. 2005-01-01 21:55:59 -0800
 * has all of the information ISO 8601 requires,
 * but the format has more whitespace and is more human-read/editable
 * -- and most importantly, php 4's strtotime() knows how to parse it.
 *
 * Thus, this format is very useful for storing in files and such
 * when not using MySQL's datetime and timestamp facilities
 */
function datetimeFormatAlmostIso8601($time)
{
    return date('Y-m-d H:i:s O');
}

/*****
echo datetimeFormatIso8601( time() )  ."\n";

echo time() ."\n";
echo datetimeFormatAlmostIso8601( time() ) ."\n";
echo strtotime( datetimeFormatAlmostIso8601( time() ))  ."\n";
exit;
*****/

/**
 * MySQL knows how to parse these buggers.
 * e.g. UNIX_TIMESTAMP('2005-01-02 01:21:38')
 */
function mysqldate($timestampToBeFormatted)
{
    return date('Y-m-d H:i:s', $timestampToBeFormatted);
}

/**
 * @param $dt: a mysql-formatted timestamp: YYYYMMDDhhmm
 * @return: a unix timestamp
 */
function mysql_timestamp_to_timestamp($dt)
{

    $yr=strval(substr($dt,0,4));
    $mo=strval(substr($dt,4,2));
    $da=strval(substr($dt,6,2));
    $hr=strval(substr($dt,8,2));
    $mi=strval(substr($dt,10,2));
    //$se=strval(substr($dt,12,2));

    return mktime ($hr,$mi,0,$mo,$da,$yr);
} 

?>