<?

/**
 * Creates a directory with the mode settings from conf.php
 * recursively creates parent dirs if they don't exist.
 * THIS IS THE PREFERRED WAY to create directories!
 * 
 */
function mkdir_default_mode($pathname)
{
    global $CONF;
    mkdir_actual_mode($pathname, $CONF['DEFAULT_MODE']);
}

/**
 * Just like php's builtin mkdir(), but ignores current umask so you actually
 * get the mode you ask for (see comments on php.net/mkdir)
 *
 * recursively creates parent dirs if they don't exist.
 */
function mkdir_actual_mode($pathname, $mode=0777)
{
        $oldumask = umask(0);
        mkdirr($pathname, $mode);
        umask($oldumask);   
}


/**
 * Create a directory structure recursively
 *
 * @author      Aidan Lister <aidan@php.net>
 * @version     1.0.0
 * @param       string   $pathname    The directory structure to create
 * @return      bool     Returns TRUE on success, FALSE on failure
 * from http://aidan.dotgeek.org/lib/?file=function.mkdirr.php
 */

function mkdirr($pathname, $mode = 0777)
{
    // Check if directory already exists
    if (is_dir($pathname) || empty($pathname)) {
        return true;
    }

    // Ensure a file does not already exist with the same name
    if (is_file($pathname)) {
        trigger_error('mkdirr() File exists', E_USER_WARNING);
        return false;
    }

    // Crawl up the directory tree
    $next_pathname = substr($pathname, 0, strrpos($pathname, DIRECTORY_SEPARATOR));
    if (mkdirr($next_pathname, $mode)) {
        if (!file_exists($pathname)) {
            return mkdir($pathname, $mode);
        }
    }
    return false;
}


//file_get_contents exists only on php >= 4.3
// so make it for us if we need it. php.net/file-get-contents
if (!function_exists('file_get_contents')) {
    function file_get_contents($file) {
        $v = file($file);
        return ($v) ? implode('', $v) : false;
    }
}


/**
 * Copy a file, or recursively copy a folder and its contents
 * adapted from http://aidan.dotgeek.org/lib/?file=function.copyr.php
 *
 * @author      Aidan Lister <aidan@php.net>
 * @version     1.0.1
 * @param       string   $source    Source path
 * @param       string   $dest      Destination path
 * @param       octal    $mode      permissions mode for NEW files
 * @return      bool     Returns TRUE on success, FALSE on failure
 */
function copyr($source, $dest, $mode)
{
    //If you're hacking on CVS version, you don't want these
//     echo $source . ":" .$dest."<BR>";
    if(preg_match('/\/CVS/', $source)) {
        return false;
    }
    // Simple copy for a file
    if (is_file($source)) {
        $success = copy($source, $dest);
        if($success) {
//            echo "chmodding $dest\n";
            chmod($dest, $mode);
        }
        return $success;
    }

    // Make destination directory
    if (!is_dir($dest)) {
//        echo "am making dest: $dest<BR>";
        mkdir_actual_mode($dest, $mode);
    }

    // Loop through the folder
    $dir = dir($source);
    while (false !== $entry = $dir->read()) {
        // Skip pointers
        if ($entry == '.' || $entry == '..') {
            continue;
        }

        // Deep copy directories
        if ($dest !== "$source/$entry") {
            copyr("$source/$entry", "$dest/$entry", $mode);
        }
    }

    // Clean up
    $dir->close();
    return true;
}

?>
