<?

/* Functions to work with javascript generation.
 */

/**
 * this function should be superceded by toJS()
 */
function assoc_array_to_js($a) {
    $js_hash = "{";
    $js_items = Array();
    foreach($a as $k => $v) {
        array_push($js_items, "'$k':'$v'");
    }
    $js_hash .= implode(",", $js_items) . "}";
    return $js_hash;
}

/**
 * The awesome function.  Any php variable gets transformed into a javascript
 * expression, ready to be evaluated as a legal javascript value!
 * so, from php:
 *   echo "a = " . toJS($bigNestedArray) . ";";
 * produces a valid javascript statement!
 * Supported types: arrays as lists, arrays as associative arrays,
 *   strings, numbers, booleans
 */
function toJS($p, $force_type='') {
    if (is_assoc_array($p) || $force_type=='assoc_array') {
        $items = array();
        foreach($p as $k=>$v) {
            $items[] = "$k : " .toJS($v);
        }
        return   "{" .implode(",\n", $items). "}\n";
    } else if (is_array($p) || $force_type=='seq_array') {
        ##should be numerical...
        $items = array();
        foreach($p as $item) {
            $items[] = toJS($item);
        }
        return "[" .implode(",", $items). "]";
    } else if (is_string($p)) {
        return "\"".addslashes($p)."\"";
    } else if (is_bool($p)) {
        return ($p) ? "true" : "false";
    } else if(is_float($p) || is_integer($p) || is_real($p)) {
        return "$p";
    } else {
        
        trigger_error("toJS() got unknown type for \$p",E_USER_ERROR);
    }
    
}

if(isset($_REQUEST['debug'])) {
    include "Decisions.php";
    echo "<PRE>".htmlspecialchars( toJS($pollingMethods) )."</PRE>";
}

/* from php.net/is_array user comment
 ****************************************************************
 * is_assoc_array tries to decide whether or not a given array   *
 * is an associative array, or a sequential array. Of course, no *
 * such distinction is made by PHP, so it really just tests      *
 * whether or not a given array could possibly be a sequential   *
 * array. Since an associative array with sequential, integer    *
 * keys 'looks' just like a sequential array, this function will *
 * be fooled.                                                    *
 *                                                               *
 * BUG: Associative arrays with sequential, integer keys 'look'  *
 * just like sequential arrays, and will be identified as such.  *
 *                                                               *
 ****************************************************************/
function is_assoc_array( $php_val ) {
    if( !is_array( $php_val ) ){
        # Neither an associative, nor non-associative array.
        return false;
    }

    $given_keys = array_keys( $php_val );
    $non_assoc_keys = range( 0, count( $php_val ) );

    if( function_exists( 'array_diff_assoc' ) ) {
        # PHP > 4.3.0
        if( array_diff_assoc( $given_keys, $non_assoc_keys ) ){
            return true;
        }
        else {
            return false;
        }
    }
    else {
        if( array_diff( $given_keys, $non_assoc_keys ) and array_diff( $non_assoc_keys, $given_keys ) ){
            return true;
        }
        else {
            return false;
        }
    }
}


            
        


?>
