#!/usr/bin/php -q
<?
// Run this very oftenly.  For example, from /etc/crontab or crontab -e:
// * * * * * /var/www/html/deme/meta/cron.php
// [that's every minute!]
//
// Make sure this file is executable:  chmod a+x meta/cron.php
// And that the first line contains the path to your php executable,
// which you can find via "which php".
//
// TODO should make this web-runnable via "wget -O /dev/null" on cronjob
//     we shouldn't require the standablone php interper to be available
//     This will require redirecting stdout to a log, not to the browser
//     since anyone could be accessing cron.php from the web...
//

$NO_CACHE_STUFF=true;
require_once "../lib/common.php";
require_once "../lib/dbinit.php";
require_once "../lib/fileutil.php";
require_once "../lib/dateutil.php";
require_once "../lib/digest.php";
require_once "../lib/Poll.php";
require_once "../lib/Group.php";


if( isset($_SERVER['REQUEST_URI'])) {
    //Uhoh, we're being run from the web -- not OK because all the stdout
    //error logging will go to the browser -- a security risk!

    error_log("cron.php: uhoh, I'm being run from the web!  Exiting.");
    exit();
}

function makeTimestampFilename($stampname)
{
    global $CONF;
    return $CONF['DEME_DATA_DIR']."/".$stampname. ".timestamp";
}

function getTimestamp($stampname)
{
    $timestamp_file = makeTimestampFilename($stampname);
    if( ! file_exists($timestamp_file))
        return 0;
    

    $formatted_time = file_get_contents($timestamp_file);
    if( $formatted_time == null )
        return 0;

    return  strtotime($formatted_time);
}

function setTimestamp($stampname, $time)
{
    $filename = makeTimestampFilename($stampname);
    $fp = fopen($filename ,'w');
    fwrite($fp, datetimeFormatAlmostIso8601($time));
    fclose($fp);
}


/**
 * @param $time: a unix timestamp
 * @return: the unix timestamp of the last occurring midnight
 *   If now is 2am,     LastMidnight was 2 hours ago.
 *   If now is 11:59pm, LastMidnight was 23hrs and 59 min ago.
 */
function getLastMidnight($time)
{
    $a = getdate($time);

    //First 3 args are hour, minute, second
    return mktime(0,0,0, $a['mon'], $a['mday'], $a['year']);
}

// Test code.
// $time = time();
// echo "now: $time\n";
// echo "now: ".strftime("%c", $time) ."\n";
// echo "last midnight: ".getLastMidnight($time) ."\n";
// echo "last midnight: ".strftime("%c", getLastMidnight($time)) ."\n";
// exit;

function main()
{
    global $CONF;
    $metadb = dbinit_meta_connect();


    $last_cron = getTimestamp("last_cron");
    $now = time();
    setTimestamp("last_cron", $now);


//     echo "last_cron: ".strftime("%c", $last_cron)."\n";
//     echo "now: ".strftime("%c", $now)."\n";

    // If now is 2am,     lastMidnight was 2 hours ago.
    // If now is 11:59pm, lastMidnight was 23hrs and 59 min ago.
    $last_midnight = getLastMidnight($now);

    //If midnight has passed since the last run... (or if it's midnight now)
    // (this should handle the special case of cron.php being run multiple
    //  times in one second at midnight)
    if($last_cron < $last_midnight) {
        $newday = true;
    } else {
        $newday = false;
    }

    
    $grouprows = $metadb->getAll("SELECT * FROM groups");
    foreach($grouprows as $row) {
        $groupname = $row['name'];
        $group = Group::fetchGroupByName($metadb, $groupname);
        if( $group == null ) {
            trigger_error("error couldn't construct group '$groupname'");
            continue;
        }
        cronForGroup($group, $metadb, $newday);
    }
}

function cronForGroup($group, $metadb, $newday)
{
    global $CURGROUP;
    $CURGROUP = $group; //many files, e.g. lib/Poll.php, need it there
    
    $db = $group->getDB();
    
    $q = "SELECT poll_id FROM polls WHERE is_open AND when_closes <= NOW()";
    $pollidrows = $db->getAll($q);
    if($pollidrows) {
        echo "CRON for ".$group->getName().": closing the following polls:";
        print_r($pollidrows);
    }
    $polls = array();
    foreach ($pollidrows as $row) {
        $poll_id  = $row['poll_id'];
        $poll = new Poll;
        
        $poll->fillFromDB($db, $poll_id);
        $polls[] =& $poll;
    }
    
    foreach ($polls as $poll) {
        $poll->closePoll($db);
    }
    
    if($newday) {
        //from lib/digest.php
        groupDigest($group, $metadb, "daily");
    }
}


main();
?>
