<?
// do NOT directly access this script via the web.  it is
// executed by ../groups, a php file.

// groupsdir_handler.php

// The standard inside-group request handler .. accepts all url's
// of a virtual directory called "groups", of the form
//
//  http://..../deme/groups/Group-Short-Name/main/home.php
//         ^^^^^^^^^^^^^^^^|||||||||||||||||||||||||||||||
//          gets this file    this is fake, not on filesystem
//                            stored as $_SERVER['PATH_INFO']

// This request handler first sets up the Group object for Group-Short-Name,
//  and then remaps the request to the correct code file and executes it.

// TO MAKE NEW HANDLERS
// the API is: when this file is interpreted,
//  * by the end there exists a new global variable:
//      $CURGROUP which is a Group object.
//  * we are chdir()'d into the dir containing the wanted script
//      look out for the special cases below
//  * we include() that script
//  * oh yeah, don't forget all those special cases delineated below.

require("../lib/common.php");
require("../lib/dbinit.php");
require("../lib/Group.php");


//Virtual directory scripts that are to be run within a group.
//currently there exists only one of these.
$virtual_dirs = array( "files" );
//   others would look like:
//      main/doclibrary    for http://.../gname/main/doclib/proposal.doc
//      members            for http://.../gname/members/bob_jones

/**
 * @return array with ['needs_trailing_slash'] and ['groupname'] and
 * ['script_wanted']
 *
 */
function parseUrl() {
    //PATH_INFO looks like /groupname/main/home.php
    //    or maybe         /groupname/marea/
    $matches = array();

    if( preg_match('|^/([^/]+)$|', $_SERVER['PATH_INFO'], $matches)) {
        //we have a groupname, but needs trailing slash.
        return array('needs_trailing_slash' => true,
                     'groupname' => $matches[1],
                     'script_wanted' => null);
    }

    
    preg_match('|/(.*?)/(.*)|', $_SERVER['PATH_INFO'], $matches);

    $ret = array('groupname' => $matches[1],
                 'script_wanted' => $matches[2],
                 'needs_trailing_slash' => false);
    
    return $ret;
}

// /deme/groups
// perhaps this should redirect to some sort of list of all groups.
if( $_SERVER['PATH_INFO'] == "") {
    echo 'no path info!';
    exit;
}


// /deme/groups/
// perhaps this should redirect to some sort of list of all groups.
if( $_SERVER['PATH_INFO'] == "/" ) {
    echo 'only a slash in pathinfo!';
    exit;
}


//At this point we know there's SOMETHING after the slash,
//which we shall interp as the groupname.

$_comps = parseUrl();


// We're tolerant: if use gives us    /deme/groups/groupname
// we'll redirect them to the correct /deme/groups/groupname/
// [this is apache's default behavior for actual directories]
if( $_comps['needs_trailing_slash'] ) {
    $CURGROUP = Group::createFakeGroup($_comps['groupname']);
    $goto = $CURGROUP->getTopUrl();
    header("Location: $goto");
    
    exit;
}

// ------ Step 1: set up $CURGROUP


//Connecting to a database is case-sensitve wrt the db name.
//Should we correct the case from the url?  No, people should be forced
//to have the correct case on the url.

$_metadb = dbinit_meta_connect();

$CURGROUP = Group::fetchGroupByName($_metadb, $_comps['groupname']);

//printr($CURGROUP);

// ------ Step 2: goto and begin requested script

//printr($_comps);
//exit;

//Special case: another virtual dir *inside* the group.
//In that case we pass over control to that virtual dir's script.
foreach($virtual_dirs as $vdir) {

    //gotta be groupname/vdir  or groupname/vdir/....
    
    if( preg_match('/^'.preg_quote($vdir).'(\/|$)/',
                   $_comps['script_wanted'])) {
        $_path = dirname($vdir);
        $_script = basename($vdir);

        chdir("../$_path");  require($_script);
        exit;
    }
}


//Trailing slash: groupname/bla/  groupname/bla/foo/   etc.
if(preg_match('/\/$/', $_comps['script_wanted'])) {
    if( is_dir( "../" .$_comps['script_wanted'] )) {
        // e.g. groupname/marea/  gets groupname/marea/index.php
        // (simulates apache default behavior)
        
        $_path = $_comps['script_wanted'];
        $_script = "index.php";
    } else {
        trigger_error("unknown case -- maybe an unregistered virtual dir?");
        $_path = dirname($_comps['script_wanted']);
        $_script = basename($_comps['script_wanted']);
    }
} else {
    $_path = dirname($_comps['script_wanted']);
    $_script = basename($_comps['script_wanted']);
}

if( $_script == "" ) {
    // Special case: groupname with only a trailing slash -> main/home.php.

    $_path = "main";
    $_script = "home.php";
}

//echo "_path: $_path <BR>";   
//echo "_script: $_script <BR>";

//Special case: a subdir without the proper trailing slash.
//[URL's generated internally by Deme should NEVER make this mistake!]
//We'll be forgiving and redirect them to the subdir with the trailing slash
// (this is apache's default behavior for filesystem serving)
//e.g.  groupname/marea           <-- wrong!  but we'll redirect:
//   to groupname/marea/
//or    groupname/marea/subdir    <-- we don't have anything this deep [yet]
if(is_dir("../$_path/$_script")) {
    if($_path == ".") $pathinsert = "/";
    else $pathinsert = "/$_path/";
    $newurl = $CURGROUP->getUrlRoot().
                $pathinsert.
                $_script.
                "/";    /** The new trailing slash! **/
    if( isset($_SERVER['QUERY_STRING']))
        $newurl .= "?".$_SERVER['QUERY_STRING'];
    header("Location: $newurl");
    exit;
}
    

//We're ready to go...

chdir("../$_path");  require($_script);


?>
